package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;

import java.util.Date;
import java.util.List;

/**
 * Builder class for basic input types.
 */
public class SimpleInputBuilder<VT> extends AbstractInputBuilder<SimpleInputBuilder,VT> {

    private final String functionName;

    protected SimpleInputBuilder(InputBuilderContext builderContext, String paramName, InputType inputType, String functionName) {
        super(builderContext, inputType, paramName);
        this.functionName = functionName;
    }

    public SimpleInputBuilder setDataType(String dataType) {
        if(dataType != null)
            cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DATATYPE, dataType);
        return this;
    }

    public SimpleInputBuilder setFormatType(String formatType) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_FORMATTYPE, formatType);
        return this;
    }

    public SimpleInputBuilder setInputType(String inputType) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_INPUTTYPE, inputType);
        return this;
    }

    public SimpleInputBuilder setRounding(boolean doRound) {
        cp.setConfigParameter("rounding",doRound);
        return this;
    }

    public SimpleInputBuilder setDecimalPlaces(String mode) {
        cp.setConfigParameter("decimalPlaces",mode);
        return this;
    }

    public SimpleInputBuilder setMax(Integer max) {
        cp.setConfigParameter("max",max);
        return this;
    }

    public SimpleInputBuilder setMax(Date max) {
        cp.setConfigParameter("max",max);
        return this;
    }

    public SimpleInputBuilder setMaxLength(Integer maxLen) {
        cp.setConfigParameter("maxLength",maxLen);
        return this;
    }

    public SimpleInputBuilder setMin(Integer min) {
        cp.setConfigParameter("min",min);
        return this;
    }

    public SimpleInputBuilder setMin(Date min) {
        cp.setConfigParameter("min",min);
        return this;
    }

    public SimpleInputBuilder setMinLength(Integer minLen) {
        cp.setConfigParameter("minLength",minLen);
        return this;
    }

    public SimpleInputBuilder setList(List l) {
        cp.setConfigParameter("list",l);
        return this;
    }

    public SimpleInputBuilder setRichTextEditor(boolean rte) {
        cp.setFormattingOption("richTextEditor",rte);
        return this;
    }
}
