package net.pricefx.logic.core.model;

import net.pricefx.logic.core.Utils;
import net.pricefx.restapiclient.model.BaseMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.math.BigDecimal;

import static net.pricefx.common.api.PriceFxInterface.FIELD_ALERT_COMPARISONELEMENT;
import static net.pricefx.common.api.PriceFxInterface.FIELD_ALERT_MESSAGE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_ALERT_OPERATOR;
import static net.pricefx.common.api.PriceFxInterface.FIELD_ALERT_THRESHOLD;
import static net.pricefx.common.api.PriceFxInterface.FIELD_CALCRESULT_LABELTRANSLATIONS;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_ALLOWOVERRIDE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_CSSPROPERTIES;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_DISPLAYOPTIONS;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_ELEMENTSUFFIX;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_HIDEONNULL;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_RESULTGROUP;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_SUMMARIZE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FORMATTYPE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_USERGROUP;

public class PricingFormulaElement extends AbstractFormulaElement {
    private static final Logger log = LoggerFactory.getLogger(PricingFormulaElement.class);

    public PricingFormulaElement() {
    }

    public void setAlert(String alertType, AlertConfiguration alertConfiguration) {
        setMapValue(alertType, alertConfiguration);
    }

    public AlertConfiguration getAlertConfiguration(String alertType) {

        BaseMap alertNode = getMapValue(alertType);

        if (alertNode == null) {
            return null;
        }
        return fromObject(alertNode, AlertConfiguration.class);
    }

    public static class AlertConfiguration extends BaseMapNullable {
        public AlertConfiguration() {
        }

        public AlertConfiguration(String operator, BigDecimal threshold, String comparisonElement, String message) {
            setStringValue(FIELD_ALERT_OPERATOR, operator);
            setDecimalValue(FIELD_ALERT_THRESHOLD, threshold);
            setStringValue(FIELD_ALERT_COMPARISONELEMENT, comparisonElement);
            setStringValue(FIELD_ALERT_MESSAGE, message);
        }

        public static AlertConfiguration getAlertConfiguration(String operator, BigDecimal threshold, String comparisonElement, String message) {
            if (StringUtils.isEmpty(operator)
                    && StringUtils.isEmpty(comparisonElement)
                    && StringUtils.isEmpty(message)
                    && threshold == null) {
                return null;
            }
            return new AlertConfiguration(operator, threshold, comparisonElement, message);
        }

        public String getOperator() {
            return getStringValue(FIELD_ALERT_OPERATOR);
        }

        public BigDecimal getThreshold() {
            return getDecimalValue(FIELD_ALERT_THRESHOLD);
        }

        public String getComparisonElement() {
            return getStringValue(FIELD_ALERT_COMPARISONELEMENT);
        }

        public String getMessage() {
            return getStringValue(FIELD_ALERT_MESSAGE);
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof AlertConfiguration) {
                AlertConfiguration alertConfig = (AlertConfiguration) obj;
                return (getThreshold() == null ? alertConfig.getThreshold() == null : getThreshold().equals(alertConfig.getThreshold()))
                                && StringUtils.equals(getOperator(), alertConfig.getOperator())
                                && StringUtils.equals(getComparisonElement(), alertConfig.getComparisonElement())
                                && StringUtils.equals(getMessage(), alertConfig.getMessage());
            }
            return super.equals(obj);

        }
    }

    public Long getDisplayOptions() {
        return getLongValue(FIELD_FE_DISPLAYOPTIONS);
    }

    public void setDisplayOptions(Long displayOptions) {
        setLongValue(FIELD_FE_DISPLAYOPTIONS, displayOptions);
    }

    public String getFormatType() {
        return getStringValue(FIELD_FORMATTYPE);
    }

    public void setFormatType(String formatType) {
        setStringValue(FIELD_FORMATTYPE, formatType);
    }

    public String getElementSuffix() {
        return getStringValue(FIELD_FE_ELEMENTSUFFIX);
    }

    public void setElementSuffix(String elementSuffix) {
        setStringValue(FIELD_FE_ELEMENTSUFFIX, elementSuffix);
    }

    public Boolean getAllowOverride() {
        return Boolean.TRUE.equals(getBooleanValue(FIELD_FE_ALLOWOVERRIDE));
    }

    public void setAllowOverride(Boolean allowOverride) {
        setBooleanValue(FIELD_FE_ALLOWOVERRIDE, allowOverride);
    }

    public Boolean getSummarize() {
        return Boolean.TRUE.equals(getBooleanValue(FIELD_FE_SUMMARIZE));
    }

    public void setSummarize(Boolean summarize) {
        setBooleanValue(FIELD_FE_SUMMARIZE, summarize);
    }

    public Boolean getHideOnNull() {
        return Boolean.TRUE.equals(getBooleanValue(FIELD_FE_HIDEONNULL));
    }

    public void setHideOnNull(Boolean hideOnNull) {
        setBooleanValue(FIELD_FE_HIDEONNULL, hideOnNull);
    }

    public String getCssProperties() {
        return getStringValue(FIELD_FE_CSSPROPERTIES);
    }

    public void setCssProperties(String cssProperties) {
        setStringValue(FIELD_FE_CSSPROPERTIES, cssProperties);
    }

    public String getResultGroup() {
        return getStringValue(FIELD_FE_RESULTGROUP);
    }

    public void setResultGroup(String resultGroup) {
        setStringValue(FIELD_FE_RESULTGROUP, resultGroup);
    }

    public String getUserGroup() {
        return getStringValue(FIELD_USERGROUP);
    }

    public void setUserGroup(String userGroup) {
        setStringValue(FIELD_USERGROUP, userGroup);
    }

    public BaseMap getLabelTranslations() {
        try {
            //older pfx releases retrieve it from backend as String
            String s = getStringValue(FIELD_CALCRESULT_LABELTRANSLATIONS);
            if (StringUtils.trimToNull(s) != null) {
                return Utils.getObjectMapper().readValue(s, BaseMap.class);
            }
        } catch (IOException e) {
            log.error("Cannot convert Label translations", e);
        }
        //newer pfx releases retrieve it from backend as Map
        return getMapValue(FIELD_CALCRESULT_LABELTRANSLATIONS);
    }

    public void setLabelTranslations(BaseMap translations) {
        setMapValue(FIELD_CALCRESULT_LABELTRANSLATIONS, translations);
    }
}

